#!/bin/sh

#-------------------------------------------------------------------
#
# Name:    lpsched_adm
#
# Purpose: Stops and restarts the lp print scheduler. Does error
#          checking to make sure the scheduler was indeed stopped
#          and started.
#
#
#-------------------------------------------------------------------

DEBUG=1

lpsched_exists()
{
	# Returns number of /usr/lib/lpsched processes found by
	# /bin/ps -ef

	# ps -ef looks at all the processes
	# the grep looks for lpsched in one of the processes
	# the awk line looks for "lp" as the owner of the process
	if [ $DEBUG ] ; then
		set -xv
	fi

	schedname="lpsched"

	count=`/bin/ps -ef | \
	         /bin/grep $schedname | \
	         /usr/bin/awk '$1 == "lp"' | \
	         /bin/wc -l`
	return $count
}

#-------------------------------------------------------------------

waitfor_schedlock()
{
	# Waits for /usr/spool/lp/SCHEDLOCK to appear and then 
	# returns or returns after 5 seconds.

	if [ $DEBUG ] ; then
		set -xv
	fi
	counter=6

	while [ $counter -gt 0 ] ; do
		if [ -f /usr/spool/lp/SCHEDLOCK ] ; then
			return
		fi
		sleep 1
		counter=`expr $counter - 1`
	done

}

#-------------------------------------------------------------------

stop_lp_scheduler()
{
	if [ $DEBUG ] ; then
		set -xv
	fi
        # Calls /usr/lib/lpshut to shut the schedular
        # Returns the number of lpsched processes.
 
        /usr/lib/lpshut > /dev/null 2>&1
        sleep 1

        return 0
}
 
#-------------------------------------------------------------------

start_lp_scheduler()
{
	if [ $DEBUG ] ; then
		set -xv
	fi
	# Calls /usr/lib/lpsched up to the number of times specified
	# in counter to start the lp scheduler. 

	# Assumes that no lpsched processes exist before the function
	# is called.

	# Returns the number of lpsched processes left after the loop

	number_of_processes=0
	counter=6
	while [ $number_of_processes -eq 0 -a $counter -gt 0 ] ; do
		/usr/lib/lpsched > /dev/null 2>&1
                sleep 1
		waitfor_schedlock
		lpsched_exists
		number_of_processes=$?
		counter=`expr $counter - 1`
	done

	return $number_of_processes
}

#-------------------------------------------------------------------
#
# Main body of script
#
#-------------------------------------------------------------------

unalias exit

case "$1" in
	start )  
		start_lp_scheduler
		number_of_processes=$?
		if [ 0 -eq $number_of_processes ] ; then
			exit 1
		else
			exit 0
		fi
	;;
	stop  ) 
		stop_lp_scheduler
		number_of_processes=$?
		if [ 0 -eq $number_of_processes ] ; then
			exit 0
		else
			exit 1
		fi
	;;
	*    ) 
		echo "$0 start|stop"
		exit 1
	;;
esac
